/* Ethan Hacker
 * Home alarm system
 * ME 608 intro to mechatronics 
 */

#include <Keypad.h> // Library for keypad
#include <Wire.h>
#include <LiquidCrystal_I2C.h> //Library for I2C LCD
LiquidCrystal_I2C lcd(0x27, 16, 2); // set the LCD address to 0x27 for a 16 chars and 2 line display

// pins for IR_sensor
  int IR_receiver = 12; 
  int IR_val;

// alarm state
  bool alarm_state = 0;
  bool reset_state = 0;

//Pin set up for speaker
  const int alarm = 7;

// set up for Keypad
  String entered ;
  const byte numRows = 4;
  const byte numCols = 3;
  String Password = "1111"; // pasword on startup 
  String Password_Change = "0000"; // set of numbers needed to input to change password
  char keypressed;
  char keymap[numRows][numCols] =
{
  {'1', '2', '3'},
  {'4', '5', '6'},
  {'7', '8', '9'},
  {'*', '0', '#'}
};

byte rowPins[numRows] = {5, 4, 3, 2};
byte colPins[numCols] = {10, 9, 8};

Keypad myKeypad = Keypad(makeKeymap(keymap), rowPins, colPins, numRows, numCols); //mapping keypad


void setup() {
  Serial.begin(9600);

// set up for IR break beam
  pinMode(IR_receiver,INPUT_PULLUP);

// set up for speaker
  pinMode(alarm,OUTPUT);

// set up for LCD screen
  lcd.init();
  lcd.backlight();
  lcd.clear();
}

void loop() {
  
  lcd.setCursor(0,0);
  lcd.print("Enter Password");

// function that reads keypad and adds it to  a string
  readKeypad();

// debugging test to see if IR sensor is being read 
  IR_val=digitalRead(IR_receiver);
  Serial.print("The IR beam value is:");
  Serial.println(IR_val);

  if (IR_val == 0 && alarm_state == 1){  // Alarm is triggered when alarm state is ON and break beam is broken
    tone(alarm, 3000);
    lcd.clear();
    lcd.setCursor(0,0);
    lcd.print("Alarm on"); // prints alarm on LCD
    delay(2000);
    lcd.clear();
    Serial.println("Alarm on"); // debugging to see if entered If statement 
  }
  
  if (keypressed == '#') { // only checks if string entered matches password set when "#" has been pressed 
    if (entered == Password && alarm_state == 1) { 
      noTone(alarm); // turns off alarm
      lcd.clear();
      lcd.setCursor(0, 1);
      lcd.print("Access Granted");
      delay(2000);
      lcd.clear();
      entered = ""; // clears what was entered 
      alarm_state = 0; // changes alarm state to OFF 
    } 
    else if (entered == Password && alarm_state == 0){
         lcd.clear();
         lcd.setCursor(0,0);
         lcd.print("Arming Alarm...");
         Serial.print("Arming Alarm...");
         int countdown = 9; // 9 seconds count down before activating the alarm
          while (countdown != 0) { // Countdown to clear the break beam or to get out of the room
            lcd.setCursor(0,1);
            lcd.print(countdown);
            countdown--;
            Serial.print("Arming in:");
            Serial.println(countdown); // for 9 sec the timer will count down then arm itself
            tone(alarm, 800, 100);
            delay(1000); 
          }
          lcd.clear();
          lcd.setCursor(0,0);
          lcd.print("Alarm Armed");
         Serial.println("Alarm Armed"); // debugging to see if alarm was turned ON
         delay(2000);
         lcd.clear();
         alarm_state = 1; // changes alarm state to ON 
         entered =""; // clears what was entered 
    }

    else if (entered == Password_Change && alarm_state == 0){
      lcd.clear();
      lcd.setCursor(0,0);
      lcd.print("Reset Password");
      entered = ""; // clears out what was entered 
      while (reset_state == 0){ // while loop holds the new password until "#" is pressed 
        readKeypad();
        lcd.setCursor(0, 1);
        lcd.print(entered);
        delay(100);
         if (keypressed == '#') { // gets out of while loop
          reset_state = 1;
         }
      }
      reset_state = 0; // changes reset state to 0
      Password = entered; // changes passsword to what was entered in While Loop, (length can change)
      Serial.print(entered); // debugging shows what new password was entered
      lcd.clear();
      lcd.setCursor(0,0);
      lcd.print("Password Changed");
      lcd.setCursor(0,1);
      lcd.print("to: ");
      lcd.setCursor(3,1);
      lcd.print(entered); // displays new password on LCD
      tone(alarm, 700, 50);
      delay(100);
      tone(alarm, 700, 50);
      delay(2000);
      Serial.print("What was entered: "); // debugging shows what new password was entered
      Serial.println(entered);
      Serial.print("New Password is: "); // debugging to show if entered and password now match 
      Serial.println(Password);
      entered = ""; // clears what was entered 
      lcd.clear();
    }

    
    else { // for if wrong password was put in 
      lcd.clear();
      lcd.setCursor(0, 1);
      lcd.print("Access Denied");
      entered = ""; // clears what was entered 
      delay(2000);
      lcd.clear();
    }
  }
  
//clears the entered numbers without checking if it matches only if "*" has been pressed 
  if (keypressed == '*') {
    entered = "";
    lcd.clear();
  }
  
  lcd.setCursor(0, 1);
  lcd.print(entered); //displays what was entered on the LCD 
  delay(50);
}

// keypad function that takes input from keypad and adds it to string 
void readKeypad() { 
  keypressed = myKeypad.getKey(); // from keypad library to read keypad 
  if (keypressed != '#') { // adds any keypad press to string if "#" wasnt pressed
    String press = String(keypressed);
    entered += press;
    Serial.println(entered); // debugging to see if what was pressed and what was entered match and it adding to the string 
  }
}
